# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: cupid_task_service.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import service as _service
from google.protobuf import service_reflection
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import cupidtaskparam_pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='cupid_task_service.proto',
  package='apsara.odps.cupid.protocol',
  serialized_pb=_b('\n\x18\x63upid_task_service.proto\x12\x1a\x61psara.odps.cupid.protocol\x1a\x14\x63upidtaskparam.proto\"m\n\x0eTableInputInfo\x12\x13\n\x0bprojectName\x18\x01 \x01(\t\x12\x11\n\ttableName\x18\x02 \x01(\t\x12\x0f\n\x07\x63olumns\x18\x03 \x01(\t\x12\x10\n\x08partSpec\x18\x04 \x01(\t\x12\x10\n\x08\x62ucketId\x18\x05 \x01(\x05\"\xd4\x01\n\x12SplitTablesRequest\x12\x12\n\nlookupName\x18\x01 \x01(\t\x12\x11\n\tsplitSize\x18\x02 \x01(\x05\x12\x12\n\nsplitCount\x18\x03 \x01(\x05\x12\x43\n\x0ftableInputInfos\x18\x04 \x03(\x0b\x32*.apsara.odps.cupid.protocol.TableInputInfo\x12\x0c\n\x04type\x18\x05 \x01(\t\x12\x16\n\x0e\x61llowNoColumns\x18\x06 \x01(\x08\x12\x18\n\x10requireSplitMeta\x18\x07 \x01(\x08\"/\n\x13SplitTablesResponse\x12\x18\n\x10inputTableHandle\x18\x01 \x01(\t\"]\n\x11WriteTableRequest\x12\x12\n\nlookupName\x18\x01 \x01(\t\x12\x13\n\x0bprojectName\x18\x02 \x01(\t\x12\x11\n\ttableName\x18\x03 \x01(\t\x12\x0c\n\x04type\x18\x04 \x01(\t\"/\n\x12WriteTableResponse\x12\x19\n\x11outputTableHandle\x18\x01 \x01(\t\"\xd8\x01\n\x12\x43ommitTableRequest\x12\x12\n\nlookupName\x18\x01 \x01(\t\x12\x19\n\x11outputTableHandle\x18\x02 \x01(\t\x12\x13\n\x0bprojectName\x18\x03 \x01(\t\x12\x11\n\ttableName\x18\x04 \x01(\t\x12\x13\n\x0bisOverWrite\x18\x05 \x01(\x08\x12\x11\n\tpartSpecs\x18\x06 \x03(\t\x12\x43\n\x0f\x63ommitFileLists\x18\x07 \x03(\x0b\x32*.apsara.odps.cupid.protocol.CommitFileList\"\x15\n\x13\x43ommitTableResponse\"3\n\tTableInfo\x12\x13\n\x0bprojectName\x18\x01 \x01(\t\x12\x11\n\ttableName\x18\x02 \x01(\t\"\x8c\x01\n\x13GetTableMetaRequest\x12\x12\n\nlookupName\x18\x01 \x01(\t\x12\x38\n\ttableInfo\x18\x02 \x01(\x0b\x32%.apsara.odps.cupid.protocol.TableInfo\x12\x13\n\x0bneedContent\x18\x03 \x01(\x08\x12\x12\n\nuploadFile\x18\x04 \x01(\t\"O\n\x14GetTableMetaResponse\x12\x1a\n\x12getTableMetaHandle\x18\x01 \x01(\t\x12\x1b\n\x13getTableMetaContent\x18\x02 \x01(\t\"I\n\x18\x43loseOutputHandleRequest\x12\x12\n\nlookupName\x18\x01 \x01(\t\x12\x19\n\x11outputTableHandle\x18\x02 \x01(\t\"\x1b\n\x19\x43loseOutputHandleResponse\"\'\n\tClusterKv\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t\"N\n\x1bPutOrCreateClusterKvRequest\x12\x13\n\x0bprojectName\x18\x01 \x01(\t\x12\x0b\n\x03key\x18\x02 \x01(\t\x12\r\n\x05value\x18\x03 \x01(\t\"\x1e\n\x1cPutOrCreateClusterKvResponse\":\n\x16\x44\x65leteClusterKvRequest\x12\x13\n\x0bprojectName\x18\x01 \x01(\t\x12\x0b\n\x03key\x18\x02 \x01(\t\"\x19\n\x17\x44\x65leteClusterKvResponse\"7\n\x13GetClusterKvRequest\x12\x13\n\x0bprojectName\x18\x01 \x01(\t\x12\x0b\n\x03key\x18\x02 \x01(\t\"%\n\x14GetClusterKvResponse\x12\r\n\x05value\x18\x01 \x01(\t\"C\n\x1cListByPrefixClusterKvRequest\x12\x13\n\x0bprojectName\x18\x01 \x01(\t\x12\x0e\n\x06prefix\x18\x02 \x01(\t\"Y\n\x1dListByPrefixClusterKvResponse\x12\x38\n\tclusterKv\x18\x01 \x03(\x0b\x32%.apsara.odps.cupid.protocol.ClusterKv\"(\n\x17\x43upidProxyTokenResponse\x12\r\n\x05token\x18\x01 \x01(\t\"/\n\x19\x43upidProxyAppNamesRequest\x12\x12\n\ninstanceId\x18\x01 \x01(\t\"+\n\x1a\x43upidProxyAppNamesResponse\x12\r\n\x05names\x18\x01 \x03(\t2\xe7\n\n\x10\x43upidTaskService\x12n\n\x0bSplitTables\x12..apsara.odps.cupid.protocol.SplitTablesRequest\x1a/.apsara.odps.cupid.protocol.SplitTablesResponse\x12k\n\nWriteTable\x12-.apsara.odps.cupid.protocol.WriteTableRequest\x1a..apsara.odps.cupid.protocol.WriteTableResponse\x12n\n\x0b\x43ommitTable\x12..apsara.odps.cupid.protocol.CommitTableRequest\x1a/.apsara.odps.cupid.protocol.CommitTableResponse\x12\x80\x01\n\x11\x43loseOutputHandle\x12\x34.apsara.odps.cupid.protocol.CloseOutputHandleRequest\x1a\x35.apsara.odps.cupid.protocol.CloseOutputHandleResponse\x12\x89\x01\n\x14PutOrCreateClusterKv\x12\x37.apsara.odps.cupid.protocol.PutOrCreateClusterKvRequest\x1a\x38.apsara.odps.cupid.protocol.PutOrCreateClusterKvResponse\x12z\n\x0f\x44\x65leteClusterKv\x12\x32.apsara.odps.cupid.protocol.DeleteClusterKvRequest\x1a\x33.apsara.odps.cupid.protocol.DeleteClusterKvResponse\x12q\n\x0cGetClusterKv\x12/.apsara.odps.cupid.protocol.GetClusterKvRequest\x1a\x30.apsara.odps.cupid.protocol.GetClusterKvResponse\x12\x8c\x01\n\x15ListByPrefixClusterKv\x12\x38.apsara.odps.cupid.protocol.ListByPrefixClusterKvRequest\x1a\x39.apsara.odps.cupid.protocol.ListByPrefixClusterKvResponse\x12q\n\x0cGetTableMeta\x12/.apsara.odps.cupid.protocol.GetTableMetaRequest\x1a\x30.apsara.odps.cupid.protocol.GetTableMetaResponse\x12}\n\x12GetCupidProxyToken\x12\x32.apsara.odps.cupid.protocol.CupidProxyTokenRequest\x1a\x33.apsara.odps.cupid.protocol.CupidProxyTokenResponse\x12\x86\x01\n\x15GetCupidProxyAppNames\x12\x35.apsara.odps.cupid.protocol.CupidProxyAppNamesRequest\x1a\x36.apsara.odps.cupid.protocol.CupidProxyAppNamesResponseB B\x15\x43upidTaskServiceProto\x80\x01\x01\x88\x01\x01\x90\x01\x01')
  ,
  dependencies=[cupidtaskparam_pb2.DESCRIPTOR,])
_sym_db.RegisterFileDescriptor(DESCRIPTOR)




_TABLEINPUTINFO = _descriptor.Descriptor(
  name='TableInputInfo',
  full_name='apsara.odps.cupid.protocol.TableInputInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='projectName', full_name='apsara.odps.cupid.protocol.TableInputInfo.projectName', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='tableName', full_name='apsara.odps.cupid.protocol.TableInputInfo.tableName', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='columns', full_name='apsara.odps.cupid.protocol.TableInputInfo.columns', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='partSpec', full_name='apsara.odps.cupid.protocol.TableInputInfo.partSpec', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='bucketId', full_name='apsara.odps.cupid.protocol.TableInputInfo.bucketId', index=4,
      number=5, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=78,
  serialized_end=187,
)


_SPLITTABLESREQUEST = _descriptor.Descriptor(
  name='SplitTablesRequest',
  full_name='apsara.odps.cupid.protocol.SplitTablesRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='lookupName', full_name='apsara.odps.cupid.protocol.SplitTablesRequest.lookupName', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='splitSize', full_name='apsara.odps.cupid.protocol.SplitTablesRequest.splitSize', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='splitCount', full_name='apsara.odps.cupid.protocol.SplitTablesRequest.splitCount', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='tableInputInfos', full_name='apsara.odps.cupid.protocol.SplitTablesRequest.tableInputInfos', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='type', full_name='apsara.odps.cupid.protocol.SplitTablesRequest.type', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='allowNoColumns', full_name='apsara.odps.cupid.protocol.SplitTablesRequest.allowNoColumns', index=5,
      number=6, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='requireSplitMeta', full_name='apsara.odps.cupid.protocol.SplitTablesRequest.requireSplitMeta', index=6,
      number=7, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=190,
  serialized_end=402,
)


_SPLITTABLESRESPONSE = _descriptor.Descriptor(
  name='SplitTablesResponse',
  full_name='apsara.odps.cupid.protocol.SplitTablesResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='inputTableHandle', full_name='apsara.odps.cupid.protocol.SplitTablesResponse.inputTableHandle', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=404,
  serialized_end=451,
)


_WRITETABLEREQUEST = _descriptor.Descriptor(
  name='WriteTableRequest',
  full_name='apsara.odps.cupid.protocol.WriteTableRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='lookupName', full_name='apsara.odps.cupid.protocol.WriteTableRequest.lookupName', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='projectName', full_name='apsara.odps.cupid.protocol.WriteTableRequest.projectName', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='tableName', full_name='apsara.odps.cupid.protocol.WriteTableRequest.tableName', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='type', full_name='apsara.odps.cupid.protocol.WriteTableRequest.type', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=453,
  serialized_end=546,
)


_WRITETABLERESPONSE = _descriptor.Descriptor(
  name='WriteTableResponse',
  full_name='apsara.odps.cupid.protocol.WriteTableResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='outputTableHandle', full_name='apsara.odps.cupid.protocol.WriteTableResponse.outputTableHandle', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=548,
  serialized_end=595,
)


_COMMITTABLEREQUEST = _descriptor.Descriptor(
  name='CommitTableRequest',
  full_name='apsara.odps.cupid.protocol.CommitTableRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='lookupName', full_name='apsara.odps.cupid.protocol.CommitTableRequest.lookupName', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='outputTableHandle', full_name='apsara.odps.cupid.protocol.CommitTableRequest.outputTableHandle', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='projectName', full_name='apsara.odps.cupid.protocol.CommitTableRequest.projectName', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='tableName', full_name='apsara.odps.cupid.protocol.CommitTableRequest.tableName', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='isOverWrite', full_name='apsara.odps.cupid.protocol.CommitTableRequest.isOverWrite', index=4,
      number=5, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='partSpecs', full_name='apsara.odps.cupid.protocol.CommitTableRequest.partSpecs', index=5,
      number=6, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='commitFileLists', full_name='apsara.odps.cupid.protocol.CommitTableRequest.commitFileLists', index=6,
      number=7, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=598,
  serialized_end=814,
)


_COMMITTABLERESPONSE = _descriptor.Descriptor(
  name='CommitTableResponse',
  full_name='apsara.odps.cupid.protocol.CommitTableResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=816,
  serialized_end=837,
)


_TABLEINFO = _descriptor.Descriptor(
  name='TableInfo',
  full_name='apsara.odps.cupid.protocol.TableInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='projectName', full_name='apsara.odps.cupid.protocol.TableInfo.projectName', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='tableName', full_name='apsara.odps.cupid.protocol.TableInfo.tableName', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=839,
  serialized_end=890,
)


_GETTABLEMETAREQUEST = _descriptor.Descriptor(
  name='GetTableMetaRequest',
  full_name='apsara.odps.cupid.protocol.GetTableMetaRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='lookupName', full_name='apsara.odps.cupid.protocol.GetTableMetaRequest.lookupName', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='tableInfo', full_name='apsara.odps.cupid.protocol.GetTableMetaRequest.tableInfo', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='needContent', full_name='apsara.odps.cupid.protocol.GetTableMetaRequest.needContent', index=2,
      number=3, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='uploadFile', full_name='apsara.odps.cupid.protocol.GetTableMetaRequest.uploadFile', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=893,
  serialized_end=1033,
)


_GETTABLEMETARESPONSE = _descriptor.Descriptor(
  name='GetTableMetaResponse',
  full_name='apsara.odps.cupid.protocol.GetTableMetaResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='getTableMetaHandle', full_name='apsara.odps.cupid.protocol.GetTableMetaResponse.getTableMetaHandle', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='getTableMetaContent', full_name='apsara.odps.cupid.protocol.GetTableMetaResponse.getTableMetaContent', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1035,
  serialized_end=1114,
)


_CLOSEOUTPUTHANDLEREQUEST = _descriptor.Descriptor(
  name='CloseOutputHandleRequest',
  full_name='apsara.odps.cupid.protocol.CloseOutputHandleRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='lookupName', full_name='apsara.odps.cupid.protocol.CloseOutputHandleRequest.lookupName', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='outputTableHandle', full_name='apsara.odps.cupid.protocol.CloseOutputHandleRequest.outputTableHandle', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1116,
  serialized_end=1189,
)


_CLOSEOUTPUTHANDLERESPONSE = _descriptor.Descriptor(
  name='CloseOutputHandleResponse',
  full_name='apsara.odps.cupid.protocol.CloseOutputHandleResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1191,
  serialized_end=1218,
)


_CLUSTERKV = _descriptor.Descriptor(
  name='ClusterKv',
  full_name='apsara.odps.cupid.protocol.ClusterKv',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='apsara.odps.cupid.protocol.ClusterKv.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='value', full_name='apsara.odps.cupid.protocol.ClusterKv.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1220,
  serialized_end=1259,
)


_PUTORCREATECLUSTERKVREQUEST = _descriptor.Descriptor(
  name='PutOrCreateClusterKvRequest',
  full_name='apsara.odps.cupid.protocol.PutOrCreateClusterKvRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='projectName', full_name='apsara.odps.cupid.protocol.PutOrCreateClusterKvRequest.projectName', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='key', full_name='apsara.odps.cupid.protocol.PutOrCreateClusterKvRequest.key', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='value', full_name='apsara.odps.cupid.protocol.PutOrCreateClusterKvRequest.value', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1261,
  serialized_end=1339,
)


_PUTORCREATECLUSTERKVRESPONSE = _descriptor.Descriptor(
  name='PutOrCreateClusterKvResponse',
  full_name='apsara.odps.cupid.protocol.PutOrCreateClusterKvResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1341,
  serialized_end=1371,
)


_DELETECLUSTERKVREQUEST = _descriptor.Descriptor(
  name='DeleteClusterKvRequest',
  full_name='apsara.odps.cupid.protocol.DeleteClusterKvRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='projectName', full_name='apsara.odps.cupid.protocol.DeleteClusterKvRequest.projectName', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='key', full_name='apsara.odps.cupid.protocol.DeleteClusterKvRequest.key', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1373,
  serialized_end=1431,
)


_DELETECLUSTERKVRESPONSE = _descriptor.Descriptor(
  name='DeleteClusterKvResponse',
  full_name='apsara.odps.cupid.protocol.DeleteClusterKvResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1433,
  serialized_end=1458,
)


_GETCLUSTERKVREQUEST = _descriptor.Descriptor(
  name='GetClusterKvRequest',
  full_name='apsara.odps.cupid.protocol.GetClusterKvRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='projectName', full_name='apsara.odps.cupid.protocol.GetClusterKvRequest.projectName', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='key', full_name='apsara.odps.cupid.protocol.GetClusterKvRequest.key', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1460,
  serialized_end=1515,
)


_GETCLUSTERKVRESPONSE = _descriptor.Descriptor(
  name='GetClusterKvResponse',
  full_name='apsara.odps.cupid.protocol.GetClusterKvResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='value', full_name='apsara.odps.cupid.protocol.GetClusterKvResponse.value', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1517,
  serialized_end=1554,
)


_LISTBYPREFIXCLUSTERKVREQUEST = _descriptor.Descriptor(
  name='ListByPrefixClusterKvRequest',
  full_name='apsara.odps.cupid.protocol.ListByPrefixClusterKvRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='projectName', full_name='apsara.odps.cupid.protocol.ListByPrefixClusterKvRequest.projectName', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='prefix', full_name='apsara.odps.cupid.protocol.ListByPrefixClusterKvRequest.prefix', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1556,
  serialized_end=1623,
)


_LISTBYPREFIXCLUSTERKVRESPONSE = _descriptor.Descriptor(
  name='ListByPrefixClusterKvResponse',
  full_name='apsara.odps.cupid.protocol.ListByPrefixClusterKvResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='clusterKv', full_name='apsara.odps.cupid.protocol.ListByPrefixClusterKvResponse.clusterKv', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1625,
  serialized_end=1714,
)


_CUPIDPROXYTOKENRESPONSE = _descriptor.Descriptor(
  name='CupidProxyTokenResponse',
  full_name='apsara.odps.cupid.protocol.CupidProxyTokenResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='token', full_name='apsara.odps.cupid.protocol.CupidProxyTokenResponse.token', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1716,
  serialized_end=1756,
)


_CUPIDPROXYAPPNAMESREQUEST = _descriptor.Descriptor(
  name='CupidProxyAppNamesRequest',
  full_name='apsara.odps.cupid.protocol.CupidProxyAppNamesRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='instanceId', full_name='apsara.odps.cupid.protocol.CupidProxyAppNamesRequest.instanceId', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1758,
  serialized_end=1805,
)


_CUPIDPROXYAPPNAMESRESPONSE = _descriptor.Descriptor(
  name='CupidProxyAppNamesResponse',
  full_name='apsara.odps.cupid.protocol.CupidProxyAppNamesResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='names', full_name='apsara.odps.cupid.protocol.CupidProxyAppNamesResponse.names', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1807,
  serialized_end=1850,
)

_SPLITTABLESREQUEST.fields_by_name['tableInputInfos'].message_type = _TABLEINPUTINFO
_COMMITTABLEREQUEST.fields_by_name['commitFileLists'].message_type = cupidtaskparam_pb2._COMMITFILELIST
_GETTABLEMETAREQUEST.fields_by_name['tableInfo'].message_type = _TABLEINFO
_LISTBYPREFIXCLUSTERKVRESPONSE.fields_by_name['clusterKv'].message_type = _CLUSTERKV
DESCRIPTOR.message_types_by_name['TableInputInfo'] = _TABLEINPUTINFO
DESCRIPTOR.message_types_by_name['SplitTablesRequest'] = _SPLITTABLESREQUEST
DESCRIPTOR.message_types_by_name['SplitTablesResponse'] = _SPLITTABLESRESPONSE
DESCRIPTOR.message_types_by_name['WriteTableRequest'] = _WRITETABLEREQUEST
DESCRIPTOR.message_types_by_name['WriteTableResponse'] = _WRITETABLERESPONSE
DESCRIPTOR.message_types_by_name['CommitTableRequest'] = _COMMITTABLEREQUEST
DESCRIPTOR.message_types_by_name['CommitTableResponse'] = _COMMITTABLERESPONSE
DESCRIPTOR.message_types_by_name['TableInfo'] = _TABLEINFO
DESCRIPTOR.message_types_by_name['GetTableMetaRequest'] = _GETTABLEMETAREQUEST
DESCRIPTOR.message_types_by_name['GetTableMetaResponse'] = _GETTABLEMETARESPONSE
DESCRIPTOR.message_types_by_name['CloseOutputHandleRequest'] = _CLOSEOUTPUTHANDLEREQUEST
DESCRIPTOR.message_types_by_name['CloseOutputHandleResponse'] = _CLOSEOUTPUTHANDLERESPONSE
DESCRIPTOR.message_types_by_name['ClusterKv'] = _CLUSTERKV
DESCRIPTOR.message_types_by_name['PutOrCreateClusterKvRequest'] = _PUTORCREATECLUSTERKVREQUEST
DESCRIPTOR.message_types_by_name['PutOrCreateClusterKvResponse'] = _PUTORCREATECLUSTERKVRESPONSE
DESCRIPTOR.message_types_by_name['DeleteClusterKvRequest'] = _DELETECLUSTERKVREQUEST
DESCRIPTOR.message_types_by_name['DeleteClusterKvResponse'] = _DELETECLUSTERKVRESPONSE
DESCRIPTOR.message_types_by_name['GetClusterKvRequest'] = _GETCLUSTERKVREQUEST
DESCRIPTOR.message_types_by_name['GetClusterKvResponse'] = _GETCLUSTERKVRESPONSE
DESCRIPTOR.message_types_by_name['ListByPrefixClusterKvRequest'] = _LISTBYPREFIXCLUSTERKVREQUEST
DESCRIPTOR.message_types_by_name['ListByPrefixClusterKvResponse'] = _LISTBYPREFIXCLUSTERKVRESPONSE
DESCRIPTOR.message_types_by_name['CupidProxyTokenResponse'] = _CUPIDPROXYTOKENRESPONSE
DESCRIPTOR.message_types_by_name['CupidProxyAppNamesRequest'] = _CUPIDPROXYAPPNAMESREQUEST
DESCRIPTOR.message_types_by_name['CupidProxyAppNamesResponse'] = _CUPIDPROXYAPPNAMESRESPONSE

TableInputInfo = _reflection.GeneratedProtocolMessageType('TableInputInfo', (_message.Message,), dict(
  DESCRIPTOR = _TABLEINPUTINFO,
  __module__ = 'cupid_task_service_pb2'
  # @@protoc_insertion_point(class_scope:apsara.odps.cupid.protocol.TableInputInfo)
  ))
_sym_db.RegisterMessage(TableInputInfo)

SplitTablesRequest = _reflection.GeneratedProtocolMessageType('SplitTablesRequest', (_message.Message,), dict(
  DESCRIPTOR = _SPLITTABLESREQUEST,
  __module__ = 'cupid_task_service_pb2'
  # @@protoc_insertion_point(class_scope:apsara.odps.cupid.protocol.SplitTablesRequest)
  ))
_sym_db.RegisterMessage(SplitTablesRequest)

SplitTablesResponse = _reflection.GeneratedProtocolMessageType('SplitTablesResponse', (_message.Message,), dict(
  DESCRIPTOR = _SPLITTABLESRESPONSE,
  __module__ = 'cupid_task_service_pb2'
  # @@protoc_insertion_point(class_scope:apsara.odps.cupid.protocol.SplitTablesResponse)
  ))
_sym_db.RegisterMessage(SplitTablesResponse)

WriteTableRequest = _reflection.GeneratedProtocolMessageType('WriteTableRequest', (_message.Message,), dict(
  DESCRIPTOR = _WRITETABLEREQUEST,
  __module__ = 'cupid_task_service_pb2'
  # @@protoc_insertion_point(class_scope:apsara.odps.cupid.protocol.WriteTableRequest)
  ))
_sym_db.RegisterMessage(WriteTableRequest)

WriteTableResponse = _reflection.GeneratedProtocolMessageType('WriteTableResponse', (_message.Message,), dict(
  DESCRIPTOR = _WRITETABLERESPONSE,
  __module__ = 'cupid_task_service_pb2'
  # @@protoc_insertion_point(class_scope:apsara.odps.cupid.protocol.WriteTableResponse)
  ))
_sym_db.RegisterMessage(WriteTableResponse)

CommitTableRequest = _reflection.GeneratedProtocolMessageType('CommitTableRequest', (_message.Message,), dict(
  DESCRIPTOR = _COMMITTABLEREQUEST,
  __module__ = 'cupid_task_service_pb2'
  # @@protoc_insertion_point(class_scope:apsara.odps.cupid.protocol.CommitTableRequest)
  ))
_sym_db.RegisterMessage(CommitTableRequest)

CommitTableResponse = _reflection.GeneratedProtocolMessageType('CommitTableResponse', (_message.Message,), dict(
  DESCRIPTOR = _COMMITTABLERESPONSE,
  __module__ = 'cupid_task_service_pb2'
  # @@protoc_insertion_point(class_scope:apsara.odps.cupid.protocol.CommitTableResponse)
  ))
_sym_db.RegisterMessage(CommitTableResponse)

TableInfo = _reflection.GeneratedProtocolMessageType('TableInfo', (_message.Message,), dict(
  DESCRIPTOR = _TABLEINFO,
  __module__ = 'cupid_task_service_pb2'
  # @@protoc_insertion_point(class_scope:apsara.odps.cupid.protocol.TableInfo)
  ))
_sym_db.RegisterMessage(TableInfo)

GetTableMetaRequest = _reflection.GeneratedProtocolMessageType('GetTableMetaRequest', (_message.Message,), dict(
  DESCRIPTOR = _GETTABLEMETAREQUEST,
  __module__ = 'cupid_task_service_pb2'
  # @@protoc_insertion_point(class_scope:apsara.odps.cupid.protocol.GetTableMetaRequest)
  ))
_sym_db.RegisterMessage(GetTableMetaRequest)

GetTableMetaResponse = _reflection.GeneratedProtocolMessageType('GetTableMetaResponse', (_message.Message,), dict(
  DESCRIPTOR = _GETTABLEMETARESPONSE,
  __module__ = 'cupid_task_service_pb2'
  # @@protoc_insertion_point(class_scope:apsara.odps.cupid.protocol.GetTableMetaResponse)
  ))
_sym_db.RegisterMessage(GetTableMetaResponse)

CloseOutputHandleRequest = _reflection.GeneratedProtocolMessageType('CloseOutputHandleRequest', (_message.Message,), dict(
  DESCRIPTOR = _CLOSEOUTPUTHANDLEREQUEST,
  __module__ = 'cupid_task_service_pb2'
  # @@protoc_insertion_point(class_scope:apsara.odps.cupid.protocol.CloseOutputHandleRequest)
  ))
_sym_db.RegisterMessage(CloseOutputHandleRequest)

CloseOutputHandleResponse = _reflection.GeneratedProtocolMessageType('CloseOutputHandleResponse', (_message.Message,), dict(
  DESCRIPTOR = _CLOSEOUTPUTHANDLERESPONSE,
  __module__ = 'cupid_task_service_pb2'
  # @@protoc_insertion_point(class_scope:apsara.odps.cupid.protocol.CloseOutputHandleResponse)
  ))
_sym_db.RegisterMessage(CloseOutputHandleResponse)

ClusterKv = _reflection.GeneratedProtocolMessageType('ClusterKv', (_message.Message,), dict(
  DESCRIPTOR = _CLUSTERKV,
  __module__ = 'cupid_task_service_pb2'
  # @@protoc_insertion_point(class_scope:apsara.odps.cupid.protocol.ClusterKv)
  ))
_sym_db.RegisterMessage(ClusterKv)

PutOrCreateClusterKvRequest = _reflection.GeneratedProtocolMessageType('PutOrCreateClusterKvRequest', (_message.Message,), dict(
  DESCRIPTOR = _PUTORCREATECLUSTERKVREQUEST,
  __module__ = 'cupid_task_service_pb2'
  # @@protoc_insertion_point(class_scope:apsara.odps.cupid.protocol.PutOrCreateClusterKvRequest)
  ))
_sym_db.RegisterMessage(PutOrCreateClusterKvRequest)

PutOrCreateClusterKvResponse = _reflection.GeneratedProtocolMessageType('PutOrCreateClusterKvResponse', (_message.Message,), dict(
  DESCRIPTOR = _PUTORCREATECLUSTERKVRESPONSE,
  __module__ = 'cupid_task_service_pb2'
  # @@protoc_insertion_point(class_scope:apsara.odps.cupid.protocol.PutOrCreateClusterKvResponse)
  ))
_sym_db.RegisterMessage(PutOrCreateClusterKvResponse)

DeleteClusterKvRequest = _reflection.GeneratedProtocolMessageType('DeleteClusterKvRequest', (_message.Message,), dict(
  DESCRIPTOR = _DELETECLUSTERKVREQUEST,
  __module__ = 'cupid_task_service_pb2'
  # @@protoc_insertion_point(class_scope:apsara.odps.cupid.protocol.DeleteClusterKvRequest)
  ))
_sym_db.RegisterMessage(DeleteClusterKvRequest)

DeleteClusterKvResponse = _reflection.GeneratedProtocolMessageType('DeleteClusterKvResponse', (_message.Message,), dict(
  DESCRIPTOR = _DELETECLUSTERKVRESPONSE,
  __module__ = 'cupid_task_service_pb2'
  # @@protoc_insertion_point(class_scope:apsara.odps.cupid.protocol.DeleteClusterKvResponse)
  ))
_sym_db.RegisterMessage(DeleteClusterKvResponse)

GetClusterKvRequest = _reflection.GeneratedProtocolMessageType('GetClusterKvRequest', (_message.Message,), dict(
  DESCRIPTOR = _GETCLUSTERKVREQUEST,
  __module__ = 'cupid_task_service_pb2'
  # @@protoc_insertion_point(class_scope:apsara.odps.cupid.protocol.GetClusterKvRequest)
  ))
_sym_db.RegisterMessage(GetClusterKvRequest)

GetClusterKvResponse = _reflection.GeneratedProtocolMessageType('GetClusterKvResponse', (_message.Message,), dict(
  DESCRIPTOR = _GETCLUSTERKVRESPONSE,
  __module__ = 'cupid_task_service_pb2'
  # @@protoc_insertion_point(class_scope:apsara.odps.cupid.protocol.GetClusterKvResponse)
  ))
_sym_db.RegisterMessage(GetClusterKvResponse)

ListByPrefixClusterKvRequest = _reflection.GeneratedProtocolMessageType('ListByPrefixClusterKvRequest', (_message.Message,), dict(
  DESCRIPTOR = _LISTBYPREFIXCLUSTERKVREQUEST,
  __module__ = 'cupid_task_service_pb2'
  # @@protoc_insertion_point(class_scope:apsara.odps.cupid.protocol.ListByPrefixClusterKvRequest)
  ))
_sym_db.RegisterMessage(ListByPrefixClusterKvRequest)

ListByPrefixClusterKvResponse = _reflection.GeneratedProtocolMessageType('ListByPrefixClusterKvResponse', (_message.Message,), dict(
  DESCRIPTOR = _LISTBYPREFIXCLUSTERKVRESPONSE,
  __module__ = 'cupid_task_service_pb2'
  # @@protoc_insertion_point(class_scope:apsara.odps.cupid.protocol.ListByPrefixClusterKvResponse)
  ))
_sym_db.RegisterMessage(ListByPrefixClusterKvResponse)

CupidProxyTokenResponse = _reflection.GeneratedProtocolMessageType('CupidProxyTokenResponse', (_message.Message,), dict(
  DESCRIPTOR = _CUPIDPROXYTOKENRESPONSE,
  __module__ = 'cupid_task_service_pb2'
  # @@protoc_insertion_point(class_scope:apsara.odps.cupid.protocol.CupidProxyTokenResponse)
  ))
_sym_db.RegisterMessage(CupidProxyTokenResponse)

CupidProxyAppNamesRequest = _reflection.GeneratedProtocolMessageType('CupidProxyAppNamesRequest', (_message.Message,), dict(
  DESCRIPTOR = _CUPIDPROXYAPPNAMESREQUEST,
  __module__ = 'cupid_task_service_pb2'
  # @@protoc_insertion_point(class_scope:apsara.odps.cupid.protocol.CupidProxyAppNamesRequest)
  ))
_sym_db.RegisterMessage(CupidProxyAppNamesRequest)

CupidProxyAppNamesResponse = _reflection.GeneratedProtocolMessageType('CupidProxyAppNamesResponse', (_message.Message,), dict(
  DESCRIPTOR = _CUPIDPROXYAPPNAMESRESPONSE,
  __module__ = 'cupid_task_service_pb2'
  # @@protoc_insertion_point(class_scope:apsara.odps.cupid.protocol.CupidProxyAppNamesResponse)
  ))
_sym_db.RegisterMessage(CupidProxyAppNamesResponse)


DESCRIPTOR.has_options = True
DESCRIPTOR._options = _descriptor._ParseOptions(descriptor_pb2.FileOptions(), _b('B\025CupidTaskServiceProto\200\001\001\210\001\001\220\001\001'))

_CUPIDTASKSERVICE = _descriptor.ServiceDescriptor(
  name='CupidTaskService',
  full_name='apsara.odps.cupid.protocol.CupidTaskService',
  file=DESCRIPTOR,
  index=0,
  options=None,
  serialized_start=1853,
  serialized_end=3236,
  methods=[
  _descriptor.MethodDescriptor(
    name='SplitTables',
    full_name='apsara.odps.cupid.protocol.CupidTaskService.SplitTables',
    index=0,
    containing_service=None,
    input_type=_SPLITTABLESREQUEST,
    output_type=_SPLITTABLESRESPONSE,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='WriteTable',
    full_name='apsara.odps.cupid.protocol.CupidTaskService.WriteTable',
    index=1,
    containing_service=None,
    input_type=_WRITETABLEREQUEST,
    output_type=_WRITETABLERESPONSE,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='CommitTable',
    full_name='apsara.odps.cupid.protocol.CupidTaskService.CommitTable',
    index=2,
    containing_service=None,
    input_type=_COMMITTABLEREQUEST,
    output_type=_COMMITTABLERESPONSE,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='CloseOutputHandle',
    full_name='apsara.odps.cupid.protocol.CupidTaskService.CloseOutputHandle',
    index=3,
    containing_service=None,
    input_type=_CLOSEOUTPUTHANDLEREQUEST,
    output_type=_CLOSEOUTPUTHANDLERESPONSE,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='PutOrCreateClusterKv',
    full_name='apsara.odps.cupid.protocol.CupidTaskService.PutOrCreateClusterKv',
    index=4,
    containing_service=None,
    input_type=_PUTORCREATECLUSTERKVREQUEST,
    output_type=_PUTORCREATECLUSTERKVRESPONSE,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='DeleteClusterKv',
    full_name='apsara.odps.cupid.protocol.CupidTaskService.DeleteClusterKv',
    index=5,
    containing_service=None,
    input_type=_DELETECLUSTERKVREQUEST,
    output_type=_DELETECLUSTERKVRESPONSE,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='GetClusterKv',
    full_name='apsara.odps.cupid.protocol.CupidTaskService.GetClusterKv',
    index=6,
    containing_service=None,
    input_type=_GETCLUSTERKVREQUEST,
    output_type=_GETCLUSTERKVRESPONSE,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='ListByPrefixClusterKv',
    full_name='apsara.odps.cupid.protocol.CupidTaskService.ListByPrefixClusterKv',
    index=7,
    containing_service=None,
    input_type=_LISTBYPREFIXCLUSTERKVREQUEST,
    output_type=_LISTBYPREFIXCLUSTERKVRESPONSE,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='GetTableMeta',
    full_name='apsara.odps.cupid.protocol.CupidTaskService.GetTableMeta',
    index=8,
    containing_service=None,
    input_type=_GETTABLEMETAREQUEST,
    output_type=_GETTABLEMETARESPONSE,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='GetCupidProxyToken',
    full_name='apsara.odps.cupid.protocol.CupidTaskService.GetCupidProxyToken',
    index=9,
    containing_service=None,
    input_type=cupidtaskparam_pb2._CUPIDPROXYTOKENREQUEST,
    output_type=_CUPIDPROXYTOKENRESPONSE,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='GetCupidProxyAppNames',
    full_name='apsara.odps.cupid.protocol.CupidTaskService.GetCupidProxyAppNames',
    index=10,
    containing_service=None,
    input_type=_CUPIDPROXYAPPNAMESREQUEST,
    output_type=_CUPIDPROXYAPPNAMESRESPONSE,
    options=None,
  ),
])

CupidTaskService = service_reflection.GeneratedServiceType('CupidTaskService', (_service.Service,), dict(
  DESCRIPTOR = _CUPIDTASKSERVICE,
  __module__ = 'cupid_task_service_pb2'
  ))

CupidTaskService_Stub = service_reflection.GeneratedServiceStubType('CupidTaskService_Stub', (CupidTaskService,), dict(
  DESCRIPTOR = _CUPIDTASKSERVICE,
  __module__ = 'cupid_task_service_pb2'
  ))


# @@protoc_insertion_point(module_scope)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright 1999-2022 Alibaba Group Holding Ltd.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from operator import itemgetter

from ... import utils, models, compat
from ..utils import is_source_collection
from .expressions import *
from .collections import SortedColumn
from .reduction import Count, GroupedCount
from .window import Window
from .merge import JoinCollectionExpr, UnionCollectionExpr


class ExprFormatter(object):
    def __init__(self, expr, indent_size=2):
        self._expr = expr

        self._ref_id = 0
        self._cache = dict()
        self._is_referred = dict()

        self._indent_size = 2

    def _next_ref_id(self):
        curr = self._ref_id
        self._ref_id += 1
        return 'ref_%s' % curr

    def _format_source_collection(self, expr, buf=None, indent=0):
        expr_id = utils.get_id(expr)
        if expr_id in self._cache:
            return self._cache[expr_id]

        ref_id = self._next_ref_id()

        need_cache = buf is None
        if need_cache:
            buf = six.StringIO()
            buf.write('Collection: {0}\n'.format(ref_id))

            indent = self._indent_size

        if isinstance(expr._source_data, models.Table):
            buf.write(utils.indent(repr(expr._source_data), indent))

        if need_cache:
            self._cache[expr_id] = ref_id, buf.getvalue()
            return self._cache[expr_id]
        else:
            return buf.getvalue()

    def _format_transformed_collection(self, expr, buf=None, indent=0):
        expr_id = utils.get_id(expr)
        if expr_id in self._cache:
            return self._cache[expr_id]

        table_aliases = dict()
        if isinstance(expr, (JoinCollectionExpr, UnionCollectionExpr)):
            lhs, rhs = expr.lhs, expr.rhs
            for s in (lhs, rhs):
                ref_id, _ = self._format_collection(s)
                table_aliases[s] = ref_id
        elif isinstance(expr, LateralViewCollectionExpr):
            colls = [expr.input] + (expr.lateral_views or [])
            for coll in colls:
                ref_id, _ = self._format_collection(coll)
                table_aliases[coll] = ref_id
        else:
            input = expr.input
            input_ref_id, _ = self._format_collection(input)
            table_aliases[input] = input_ref_id

        ref_id = self._next_ref_id()

        need_cache = buf is None
        if need_cache:
            buf = six.StringIO()
            buf.write('Collection: {0}\n'.format(ref_id))

            indent = self._indent_size

        buf.write(utils.indent('{0}[collection]\n'.format(expr.node_name), indent))
        for name, node in expr.iter_args():
            if name.startswith('collection'):
                content = '{0}: {1}\n'.format(name, table_aliases[node])
            else:
                if node is None:
                    continue
                elif name.startswith('lateral_view'):
                    content = '{0}:\n{1}\n'.format(name, utils.indent('\n'.join(table_aliases[lv] for lv in node),
                                                                      self._indent_size))
                else:
                    content = '{0}:\n'.format(name)
            buf.write(utils.indent(content, indent+self._indent_size))
            if isinstance(node, list):
                for n in node:
                    self._format_node(n, buf, indent=indent+self._indent_size*2)
            else:
                self._format_node(node, buf, indent=indent+self._indent_size*2)

        if need_cache:
            self._cache[expr_id] = ref_id, buf.getvalue()
            return self._cache[expr_id]
        else:
            return buf.getvalue()

    def _format_collection(self, expr, buf=None, indent=0):
        if is_source_collection(expr) or type(expr) is CollectionExpr:
            return self._format_source_collection(expr, buf=buf, indent=indent)
        else:
            return self._format_transformed_collection(expr, buf=buf, indent=indent)

    def _format_column_content(self, expr):
        input_ref_id, _ = self._format_collection(expr._input)

        return "{0} = Column[{1}] '{2}' from collection {3}\n".format(
            expr.name, expr.output_type(), expr.source_name, input_ref_id)

    def _format_column(self, expr, buf, indent=0):
        content = self._format_column_content(expr)
        buf.write(utils.indent(content, indent))

    def _format_count_table(self, expr, buf, indent=0):
        input_ref_id, _ = self._format_collection(expr.input)

        buf.write(utils.indent("{0} = {1}[{2}]\n".format(
                expr.name, expr.node_name, expr.output_type()), indent))
        buf.write(utils.indent("collection: {0}\n".format(input_ref_id), indent+self._indent_size))

    def _format_sorted_column(self, expr, buf, indent=0):
        if isinstance(expr.input, Column):
            col = expr.input
            input_ref_id, _ = self._format_collection(expr.input.input)
        else:
            col = expr
            input_ref_id, _ = self._format_collection(expr._input)

        contents = ['SortKey', utils.indent('by', self._indent_size),
                    utils.indent(self._format_column_content(col), self._indent_size*2).rstrip('\n'),
                    utils.indent('ascending', self._indent_size),
                    utils.indent(str(expr._ascending), self._indent_size*2)
                    ]
        buf.write(utils.indent('\n'.join(contents+['']), indent))

    def _format_window(self, expr, buf, indent=0):
        if self._need_output_seq_name(expr, indent=indent):
            buf.write(utils.indent('{0} = '.format(expr.name), indent))
        buf.write(utils.indent('{0}[{1}]\n'.format(expr.node_name, expr.output_type()), indent))

        kw = list(expr.iter_args())

        arg = kw[0][1]
        if isinstance(arg, CollectionExpr):
            self._format_collection(arg)
        else:
            self._format_node(arg, buf, indent=indent+self._indent_size)

        for name, node in kw[1:]:
            if node is None:
                continue
            content = '{0}:\n'.format(name)

            buf.write(utils.indent(content, indent+self._indent_size))
            if isinstance(node, (list, tuple)):
                for n in node:
                    self._format_node(n, buf, indent=indent+self._indent_size*2)
            else:
                self._format_node(node, buf, indent=indent+self._indent_size*2)

    @classmethod
    def _need_output_seq_name(cls, expr, indent=0):
        if indent == 0 and expr.name:
            return True

        if expr.source_name is not None and expr.name is not None and \
                expr.source_name != expr.name:
            return True

    def _format_sub_node(self, expr, buf, indent=0):
        for name, node in expr.iter_args():
            if name.startswith('_'):
                self._format_node(node, buf, indent=indent+self._indent_size)
                continue
            if node is None:
                continue
            content = '{0}:\n'.format(name)

            buf.write(utils.indent(content, indent+self._indent_size))
            if isinstance(node, (list, tuple)):
                for n in node:
                    self._format_node(n, buf, indent=indent+self._indent_size*2)
            else:
                self._format_node(node, buf, indent=indent+self._indent_size*2)

    def _format_node(self, expr, buf, indent=0):
        if isinstance(expr, SortedColumn):
            self._format_sorted_column(expr, buf, indent=indent)
        elif isinstance(expr, Column):
            self._format_column(expr, buf, indent=indent)
        elif isinstance(expr, Window):
            self._format_window(expr, buf, indent=indent)
        elif isinstance(expr, CollectionExpr):
            self._format_collection(expr, buf, indent)
        elif isinstance(expr, (Count, GroupedCount)) and isinstance(expr.input, CollectionExpr):
            self._format_count_table(expr, buf, indent=indent)
        elif isinstance(expr, SequenceExpr):
            txt = '{0}[{1}]\n'.format(expr.node_name, expr.output_type())
            if self._need_output_seq_name(expr, indent=indent):  # TODO: need to check
                txt = '{0} = {1}'.format(expr.name, txt)
            buf.write(utils.indent(txt, indent))
            self._format_sub_node(expr, buf, indent=indent)
        elif isinstance(expr, Scalar) and getattr(expr, '_input', None) is not None:
            buf.write(utils.indent(
                '{0} = {1}[{2}]\n'.format(
                    expr.node_name.lower(), expr.node_name, expr.output_type()), indent))
            self._format_sub_node(expr, buf, indent=indent)
        elif isinstance(expr, Scalar):
            buf.write(utils.indent(expr.output_type(), indent))
            buf.write('\n')
            buf.write(utils.indent(repr(expr._value)+'\n', indent+self._indent_size))

    def format(self):
        buf = six.StringIO()
        self._format_node(self._expr, buf)

        result = six.StringIO()
        collections = sorted(compat.lvalues(self._cache), key=itemgetter(0))
        result.write('\n'.join([it[1] for it in collections]))
        result.write('\n')
        result.write(buf.getvalue())

        return result.getvalue()

    __call__ = format



# encoding: utf-8
# Copyright 1999-2022 Alibaba Group Holding Ltd.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from functools import partial
import json

from ..utils import ML_ARG_PREFIX
from ..enums import FieldRole, FieldContinuity
from ...compat import six
from ...df.expr.core import Node
from ...df.backends.context import context
from ...df.expr.collections import FilterPartitionCollectionExpr

"""
Common exporters
"""


def repr_exist(obj):
    return repr(obj) if obj else None


def get_ml_input(expr, input_name):
    input_expr = getattr(expr, ML_ARG_PREFIX + input_name, None)
    if isinstance(input_expr, Node):
        return input_expr
    else:
        return None


def get_input_table_name(expr, input_name):
    input_expr = get_ml_input(expr, input_name)
    if input_expr is None:
        return None
    if isinstance(input_expr, FilterPartitionCollectionExpr) and not context.is_cached(input_expr):
        try:
            ds = next(input_expr.data_source())
        except StopIteration:
            return None
    else:
        ds = input_expr._source_data
    if ds is None:
        return None
    else:
        return '{0}.{1}'.format(ds.project.name, ds.name)


def get_input_partitions(expr, input_name):
    input_expr = get_ml_input(expr, input_name)
    if input_expr is None:
        return None
    if not isinstance(input_expr, FilterPartitionCollectionExpr) or context.is_cached(input_expr):
        return None
    predicate = input_expr._predicate_string

    def _quote_remover(filter_str):
        parts = filter_str.split('=', 1)
        if parts[1].startswith("'") and parts[1].endswith("'"):
            parts[1] = parts[1][1:-1]
        return '='.join(parts)

    return ','.join('/'.join([_quote_remover(s) for s in p.split(',')]) for p in predicate.split('/'))


get_input_table_partitions = get_input_partitions


def get_input_model_table_name(expr, input_name, table_name):
    obj = get_ml_input(expr, input_name)
    if not getattr(obj, '_source_data', None):
        return None
    tm = obj._source_data
    table_name = tm.tables[table_name].name
    if tm.project.name == tm.odps.project:
        return table_name
    else:
        return '{0}.{1}'.format(tm.project.name, table_name)


def get_input_model_param(expr, input_name, param_name):
    if expr._params.get(param_name, None):
        return expr._params[param_name]
    obj = get_ml_input(expr, input_name)
    if obj is None:
        return None
    return obj._model_params[param_name]


def get_output_model_table_name(expr, output_name, table_name):
    expr = expr.outputs().get(output_name)
    if expr is None or not hasattr(expr, table_name):
        return None
    persist_kw = getattr(expr, 'persist_kw', dict())
    model_name = persist_kw.get('_model')
    table_name = getattr(expr, table_name).table_name(model_name)
    if persist_kw.get('_project') is None:
        return table_name
    else:
        return persist_kw.get('_project') + '.' + table_name


def get_output_table_name(expr, output_name):
    if hasattr(expr, 'tables'):
        return getattr(expr.tables, output_name)

    expr_output = expr.outputs().get(output_name)
    if expr_output is None:
        if 'required_outputs' in expr.shared_kw:
            return expr.shared_kw['required_outputs'].get(output_name)
        else:
            return None
    persist_kw = getattr(expr_output, 'persist_kw', dict())
    if persist_kw.get('_project') is None:
        return persist_kw.get('_table')
    else:
        return persist_kw.get('_project') + '.' + persist_kw.get('_table')


def get_output_table_partition(expr, output_name):
    expr = expr.outputs().get(output_name)
    if expr is None:
        return None
    return repr_exist(getattr(expr, 'persist_kw', dict()).get('_partition'))

get_output_table_partitions = get_output_table_partition


def get_input_model_name(expr, input_name):
    model_expr = get_ml_input(expr, input_name)
    model_obj = getattr(model_expr, '_source_data', None)
    rest_client = getattr(model_obj, "_client", None)
    client_project = getattr(rest_client, "project", None)
    if model_obj is None:
        return None
    elif model_obj.project.name == client_project:
        return model_obj.name
    else:
        return model_obj.project.name + '.' + model_obj.name


def get_output_model_name(expr, output_name):
    expr = expr.outputs().get(output_name)
    if expr is None:
        return None
    persist_kw = getattr(expr, 'persist_kw', dict())
    if persist_kw.get('_project') is None:
        return persist_kw.get('_model')
    else:
        return persist_kw.get('_project') + '.' + persist_kw.get('_model')


def get_input_field_names(expr, param_name, input_name, field_role=None, field_func=None):
    if expr._params.get(param_name, None):
        return expr._params[param_name]
    data_obj = get_ml_input(expr, input_name)
    if data_obj is None:
        return None
    fields = data_obj._ml_fields
    if field_role:
        return [f.name for f in fields if field_role in f.role]
    else:
        return [f.name for f in fields if field_func(f)]


def get_input_field_name(expr, param_name, input_name, field_role=None, field_func=None):
    v = get_input_field_names(expr, param_name, input_name, field_role, field_func)
    return v[0] if v else None


def get_input_field_ids(expr, param_name, input_name, field_role=None, field_func=None):
    if expr._params.get(param_name, None):
        existing_values = expr._params[param_name]
        if isinstance(existing_values, six.string_types):
            existing_values = set(v.strip() for v in existing_values.split(','))
    else:
        existing_values = set()
    data_obj = get_ml_input(expr, input_name)
    if data_obj is None:
        return None
    fields = data_obj._ml_fields
    if existing_values:
        return [idx for idx, f in enumerate(fields) if f.name in existing_values]
    if field_role:
        return [idx for idx, f in enumerate(fields) if field_role in f.role]
    else:
        return [idx for idx, f in enumerate(fields) if field_func(f)]


def get_input_field_id(expr, param_name, input_name, field_role=None, field_func=None):
    v = get_input_field_ids(expr, param_name, input_name, field_role, field_func)
    return v[0] if v else None


generate_model_name = get_output_model_name
get_group_id_column = partial(get_input_field_names, field_role=FieldRole.GROUP_ID)
get_label_column = partial(get_input_field_names, field_role=FieldRole.LABEL)
get_weight_column = partial(get_input_field_names, field_role=FieldRole.WEIGHT)
get_predicted_class_column = partial(get_input_field_names, field_role=FieldRole.PREDICTED_CLASS)
get_predicted_score_column = partial(get_input_field_names, field_role=FieldRole.PREDICTED_SCORE)
get_predicted_detail_column = partial(get_input_field_names, field_role=FieldRole.PREDICTED_DETAIL)
get_feature_columns = partial(get_input_field_names, field_role=FieldRole.FEATURE)
get_non_feature_columns = partial(get_input_field_names, field_func=lambda f: FieldRole.FEATURE not in f.role)


def get_feature_continuous(expr, param_name, input_name):
    if expr._params.get(param_name, None):
        return expr._params[param_name]
    return [1 if v == FieldContinuity.CONTINUOUS else 0
            for v in get_input_field_continuous(expr, input_name, FieldRole.FEATURE)]


def get_input_field_continuous(expr, input_name, field_role):
    data_obj = get_ml_input(expr, input_name)
    if data_obj is None:
        return None
    fields = data_obj._ml_fields
    return [f.continuity for f in fields if field_role in f.role]


def get_original_columns(expr, input_name):
    data_obj = get_ml_input(expr, input_name)
    if data_obj is None:
        return None
    fields = data_obj._ml_fields
    return [f.name for f in fields if not f.is_append]


def get_unique_feature_field_property(expr, param_name, input_name, prop_fetcher, default=None):
    if expr._params.get(param_name, None):
        return expr._params[param_name]
    data_obj = get_ml_input(expr, input_name)
    if data_obj is None:
        return None
    fields = data_obj._ml_fields
    feature_fields = [f for f in fields if FieldRole.FEATURE in f.role]
    value_set = set(prop_fetcher(f) for f in feature_fields)
    if len(value_set) == 0:
        return default
    if len(value_set) != 1:
        raise ValueError('Property %s must be unique among features.' % param_name)
    ret = next(iter(value_set))
    return ret if ret is not None else default


def convert_json(expr, param_name):
    param_val = expr._params.get(param_name, None)
    if isinstance(param_val, six.string_types):
        return param_val
    elif param_val:
        return json.dumps(expr._params[param_name], separators=(',', ':'))
    return None


get_kv_delimiter = partial(get_unique_feature_field_property,
                           prop_fetcher=lambda f: f.kv_config.kv_delimiter if f.kv_config else None)
get_item_delimiter = partial(get_unique_feature_field_property,
                             prop_fetcher=lambda f: f.kv_config.item_delimiter if f.kv_config else None)
get_enable_sparse = partial(get_unique_feature_field_property,
                            prop_fetcher=lambda f: True if f.kv_config is not None else None)


def get_sparse_predict_feature_columns(expr, param_name, input_name, enable_sparse_param='enableSparse'):
    return get_feature_columns(expr, param_name, input_name)\
        if get_enable_sparse(expr, enable_sparse_param, input_name) else None



# !/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright 1999-2022 Alibaba Group Holding Ltd.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging

from mars.deploy.kubernetes.scheduler import K8SSchedulerApplication

from .core import CupidServiceMixin, CupidK8SPodsIPWatcher

try:
    from mars.deploy.kubernetes.scheduler import WorkerWatcherActor

    class CupidWorkerWatcherActor(WorkerWatcherActor):
        watcher_cls = CupidK8SPodsIPWatcher

except ImportError:
    WorkerWatcherActor = CupidWorkerWatcherActor = None

logger = logging.getLogger(__name__)


class CupidSchedulerApplication(CupidServiceMixin, K8SSchedulerApplication):
    def __init__(self, *args, **kwargs):
        from mars.actors import register_actor_implementation
        from mars.scheduler import SessionManagerActor
        from odps.mars_extension.legacy.actors import CupidSessionManagerActor

        register_actor_implementation(SessionManagerActor, CupidSessionManagerActor)
        if WorkerWatcherActor is not None:
            register_actor_implementation(WorkerWatcherActor, CupidWorkerWatcherActor)
        super().__init__(*args, **kwargs)


main = CupidSchedulerApplication()

if __name__ == "__main__":
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright 1999-2022 Alibaba Group Holding Ltd.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging
import sys
import struct

from requests.exceptions import StreamConsumedError

from . import io
from .base import BaseTunnel
from .checksum import Checksum
from .errors import TunnelError
from .. import serializers, options
from ..models import errors
from ..compat import irange, Enum, six
from ..utils import to_binary, to_text

logger = logging.getLogger(__name__)

MAX_CHUNK_SIZE = 256 * 1024 * 1024
MIN_CHUNK_SIZE = 1
CHECKSUM_SIZE = 4

CHECKSUM_PACKER = '>i' if six.PY2 else '>I'


class VolumeTunnel(BaseTunnel):
    def create_download_session(self, volume, partition_spec, file_name, download_id=None, compress_option=None,
                                compress_algo=None, compress_level=None, compress_strategy=None):
        if not isinstance(volume, six.string_types):
            volume = volume.name
        volume = self._project.volumes[volume]
        if compress_option is None and compress_algo is not None:
            compress_option = io.CompressOption(
                compress_algo=compress_algo, level=compress_level, strategy=compress_strategy)

        return VolumeDownloadSession(
            self.tunnel_rest, volume, partition_spec, file_name, download_id=download_id,
            compress_option=compress_option, quota_name=self._quota_name
        )

    def create_upload_session(self, volume, partition_spec, upload_id=None, compress_option=None,
                              compress_algo=None, compress_level=None, compress_strategy=None):
        if not isinstance(volume, six.string_types):
            volume = volume.name
        volume = self._project.volumes[volume]
        if compress_option is None and compress_algo is not None:
            compress_option = io.CompressOption(
                compress_algo=compress_algo, level=compress_level, strategy=compress_strategy)

        return VolumeUploadSession(
            self.tunnel_rest, volume, partition_spec, upload_id=upload_id,
            compress_option=compress_option, quota_name=self._quota_name
        )


class VolumeFSTunnel(BaseTunnel):
    def open_reader(self, volume, path, start=None, length=None, compress_option=None, compress_algo=None,
                    compress_level=None, compress_strategy=None):
        if not isinstance(volume, six.string_types):
            volume = volume.name
        volume = self._project.volumes[volume]

        if start is None:
            start = 0
        if length is None:
            file_obj = volume[path]
            length = file_obj.length

        headers = {
            'Range': 'bytes={0}-{1}'.format(start, start + length - 1),
            'x-odps-volume-fs-path': '/' + volume.name + '/' + path.lstrip('/'),
        }

        if compress_option is not None:
            if compress_option.algorithm == io.CompressOption.CompressAlgorithm.ODPS_ZLIB:
                headers['Accept-Encoding'] = 'deflate'
            elif compress_option.algorithm != io.CompressOption.CompressAlgorithm.ODPS_RAW:
                raise TunnelError('invalid compression option')

        url = volume.resource(client=self.tunnel_rest)
        resp = self.tunnel_rest.get(url, headers=headers, stream=True)
        if not self.tunnel_rest.is_ok(resp):
            e = TunnelError.parse(resp)
            raise e

        if compress_option is None and compress_algo is not None:
            compress_option = io.CompressOption(
                compress_algo=compress_algo, level=compress_level, strategy=compress_strategy)

        content_encoding = resp.headers.get('Content-Encoding')
        if content_encoding is not None:
            compress = True
        else:
            compress = False

        option = compress_option if compress else None
        return VolumeReader(self.tunnel_rest, resp, option)

    def open_writer(self, volume, path, replication=None, compress_option=None, compress_algo=None,
                    compress_level=None, compress_strategy=None):
        if not isinstance(volume, six.string_types):
            volume = volume.name
        volume = self._project.volumes[volume]

        headers = {
            'Content-Type': 'application/octet-stream',
            'Transfer-Encoding': 'chunked',
            'x-odps-volume-fs-path': '/' + volume.name + '/' + path.lstrip('/'),
        }
        params = {}

        if compress_option is None and compress_algo is not None:
            compress_option = io.CompressOption(
                compress_algo=compress_algo, level=compress_level, strategy=compress_strategy)
        if compress_option is not None:
            if compress_option.algorithm == io.CompressOption.CompressAlgorithm.ODPS_ZLIB:
                headers['Content-Encoding'] = 'deflate'
            elif compress_option.algorithm != io.CompressOption.CompressAlgorithm.ODPS_RAW:
                raise TunnelError('invalid compression option')

        if replication:
            params['replication'] = replication

        url = volume.resource(client=self.tunnel_rest)

        chunk_upload = lambda data: self.tunnel_rest.post(url, data=data, params=params, headers=headers)
        if compress_option is None and compress_algo is not None:
            compress_option = io.CompressOption(
                compress_algo=compress_algo, level=compress_level, strategy=compress_strategy)
        return VolumeFSWriter(self.tunnel_rest, chunk_upload, volume, path, compress_option)


class VolumeDownloadSession(serializers.JSONSerializableModel):
    __slots__ = (
        '_client', 'project_name', '_compress_option', '_quota_name',
    )

    class Status(Enum):
        UNKNOWN = 'UNKNOWN'
        NORMAL = 'NORMAL'
        CLOSED = 'CLOSED'
        EXPIRED = 'EXPIRED'

    id = serializers.JSONNodeField('DownloadID')
    status = serializers.JSONNodeField(
        'Status', parse_callback=lambda v: VolumeDownloadSession.Status(v.upper())
    )
    file_name = serializers.JSONNodeField('File', 'FileName')
    file_length = serializers.JSONNodeField('File', 'FileLength')
    volume_name = serializers.JSONNodeField('Partition', 'Volume')
    partition_spec = serializers.JSONNodeField('Partition', 'Partition')

    def __init__(
        self, client, volume, partition_spec, file_name=None, download_id=None,
            compress_option=None, quota_name=None
    ):
        super(VolumeDownloadSession, self).__init__()

        self._client = client
        self._compress_option = compress_option
        self._quota_name = quota_name
        self.project_name = volume.project.name
        self.volume_name = volume.name
        self.partition_spec = partition_spec
        self.file_name = file_name

        if download_id is None:
            self._init()
        else:
            self.id = download_id
            self.reload()

        logger.info("Tunnel session created: %r", self)
        if options.tunnel_session_create_callback:
            options.tunnel_session_create_callback(self)

    def __repr__(self):
        return (
            "<VolumeDownloadSession id=%s project_name=%s volume_name=%s partition_spec=%s>"
            % (self.id, self.project_name, self.volume_name, self.partition_spec)
        )

    def resource(self, client=None, endpoint=None):
        endpoint = endpoint if endpoint is not None else (client or self._client).endpoint
        return endpoint + '/projects/%s/tunnel/downloads' % self.project_name

    def _init(self):
        headers = {'Content-Length': '0'}
        params = dict(type='volumefile', target='/'.join(
            [self.project_name, self.volume_name, self.partition_spec, self.file_name]
        ))
        if self._quota_name is not None:
            params['quotaName'] = self._quota_name

        url = self.resource()
        resp = self._client.post(url, {}, params=params, headers=headers)
        if self._client.is_ok(resp):
            self.parse(resp, obj=self)
        else:
            e = TunnelError.parse(resp)
            raise e

    def reload(self):
        headers = {'Content-Length': '0'}
        params = {}

        if self.partition_spec is not None and len(self.partition_spec) > 0:
            params['partition'] = self.partition_spec
        if self._quota_name is not None:
            params['quotaName'] = self._quota_name

        url = self.resource() + '/' + str(self.id)
        resp = self._client.get(url, params=params, headers=headers)
        if self._client.is_ok(resp):
            self.parse(resp, obj=self)
        else:
            e = TunnelError.parse(resp)
            raise e

    def open(self, start=0, length=sys.maxsize):
        compress_option = self._compress_option or io.CompressOption()

        params = {}

        headers = {'Content-Length': 0, 'x-odps-tunnel-version': 4}
        if compress_option.algorithm == io.CompressOption.CompressAlgorithm.ODPS_ZLIB:
            headers['Accept-Encoding'] = 'deflate'
        elif compress_option.algorithm != io.CompressOption.CompressAlgorithm.ODPS_RAW:
            raise TunnelError('invalid compression option')

        params['data'] = ''
        params['range'] = '(%s,%s)' % (start, length)
        if self._quota_name is not None:
            params['quotaName'] = self._quota_name

        url = self.resource()
        resp = self._client.get(url + '/' + self.id, params=params, headers=headers, stream=True)
        if not self._client.is_ok(resp):
            e = TunnelError.parse(resp)
            raise e

        content_encoding = resp.headers.get('Content-Encoding')
        if content_encoding is not None:
            if content_encoding == 'deflate':
                self._compress_option = io.CompressOption(
                    io.CompressOption.CompressAlgorithm.ODPS_ZLIB, -1, 0)
            else:
                raise TunnelError('Invalid content encoding')
            compress = True
        else:
            compress = False

        option = compress_option if compress else None
        return VolumeReader(self._client, resp, option)


class VolumeReader(object):
    def __init__(self, client, response, compress_option):
        self._client = client
        self._response = io.RequestsInputStream(response)
        self._compress_option = compress_option
        self._crc = Checksum(method='crc32')
        self._buffer_size = 0
        self._initialized = False
        self._last_line_ending = None
        self._eof = False

        # buffer part left by sized read or read-line operation, see read()
        self._left_part = None
        self._left_part_pos = 0

        # left part of checksum block when chunked, see _read_buf()
        self._chunk_left = None

    def _raw_read(self, l):
        return self._response.read(l)

    def _init_buf(self):
        size_buf = self._raw_read(4)
        if not size_buf:
            raise IOError('Tunnel reader breaks unexpectedly.')
        self._crc.update(size_buf)
        chunk_size = struct.unpack('>I', size_buf)[0]
        if chunk_size > MAX_CHUNK_SIZE or chunk_size < MIN_CHUNK_SIZE:
            raise IOError("ChunkSize should be in [%d, %d], now is %d." % (MIN_CHUNK_SIZE, MAX_CHUNK_SIZE, chunk_size))
        self._buffer_size = CHECKSUM_SIZE + chunk_size

    def _read_buf(self):
        has_stuff = False

        data_buffer = six.BytesIO()
        if self._chunk_left:
            # we have cached chunk left, add to buffer
            data_buffer.write(self._chunk_left)
            self._chunk_left = None
        while data_buffer.tell() < self._buffer_size:
            try:
                # len(buf) might be less than _buffer_size
                buf = self._raw_read(self._buffer_size)
                if not buf:
                    break
                data_buffer.write(buf)
                has_stuff = True
            except StopIteration:
                break
            except StreamConsumedError:
                break
        if not has_stuff:
            return None

        # check if we need to store the rest part.
        if data_buffer.tell() <= self._buffer_size:
            buf = data_buffer.getvalue()
        else:
            buf_all = data_buffer.getvalue()
            buf, self._chunk_left = buf_all[:self._buffer_size], buf_all[self._buffer_size:]

        if len(buf) >= CHECKSUM_SIZE:
            self._data_size = len(buf) - CHECKSUM_SIZE
            self._crc.update(buf[:self._data_size])
            checksum = struct.unpack_from(CHECKSUM_PACKER, buf, self._data_size)[0]
            if checksum != self._crc.getvalue():
                raise IOError('CRC check error in VolumeReader.')
        else:
            raise IOError('Invalid VolumeReader.')
        return bytearray(buf[:self._data_size])

    def read(self, size=None, break_line=False):
        if size is None:
            size = sys.maxsize
        if self._eof:
            return None
        if size == 0:
            return six.binary_type()

        if not self._initialized:
            self._initialized = True
            self._init_buf()

        has_stuff = False

        out_buf = six.BytesIO()
        if self._left_part:
            if break_line:
                # deal with Windows line endings
                if self._left_part[self._left_part_pos] == ord('\n') and self._last_line_ending == ord('\r'):
                    self._last_line_ending = None
                    self._left_part_pos += 1

                for idx in irange(self._left_part_pos, len(self._left_part)):
                    if self._left_part[idx] not in (ord('\r'), ord('\n')):
                        continue
                    self._last_line_ending = self._left_part[idx]
                    self._left_part[idx] = ord('\n')
                    ret = self._left_part[self._left_part_pos:idx + 1]
                    self._left_part_pos = idx + 1
                    if self._left_part_pos == len(self._left_part):
                        self._left_part = None
                        self._left_part_pos = 0
                    return bytes(ret)
            if len(self._left_part) - self._left_part_pos >= size:
                ret = self._left_part[self._left_part_pos:self._left_part_pos + size]
                self._left_part_pos += size
                return bytes(ret)
            else:
                out_buf.write(bytes(self._left_part[self._left_part_pos:]))
                self._left_part = None
                self._left_part_pos = 0
                has_stuff = True
        length_left = size - out_buf.tell()
        while length_left > 0:
            buf = self._read_buf()
            if buf is None:
                self._eof = True
                break
            has_stuff = True
            start_pos = 0
            if break_line:
                if buf[0] == ord('\n') and self._last_line_ending == ord('\r'):
                    start_pos = 1
                for idx in irange(start_pos, len(buf)):
                    if buf[idx] not in (ord('\r'), ord('\n')):
                        continue
                    self._last_line_ending = buf[idx]
                    buf[idx] = ord('\n')
                    out_buf.write(bytes(buf[start_pos:idx + 1]))
                    if idx + 1 < len(buf):
                        self._left_part = buf[idx + 1:]
                        self._left_part_pos = 0
                    return out_buf.getvalue()

            if len(buf) >= length_left:
                out_buf.write(bytes(buf[start_pos:start_pos + length_left]))
                if len(buf) > length_left:
                    self._left_part = buf[start_pos + length_left:]
                    self._left_part_pos = 0
                length_left = 0
            else:
                out_buf.write(bytes(buf[start_pos:start_pos + self._data_size]))
                length_left -= self._data_size
        return out_buf.getvalue() if has_stuff else None

    def _it(self, size=sys.maxsize, encoding='utf-8'):
        while True:
            line = self.readline(size, encoding=encoding)
            if line is None:
                break
            yield line

    def readline(self, size=sys.maxsize, encoding='utf-8'):
        line = self.read(size, break_line=True)
        return to_text(line, encoding=encoding)

    def readlines(self, size=sys.maxsize, encoding='utf-8'):
        return [line for line in self._it(size, encoding=encoding)]

    def __iter__(self):
        return self._it()

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        pass


class VolumeUploadSession(serializers.JSONSerializableModel):
    __slots__ = (
        '_client', '_compress_option', 'project_name', 'volume_name',
        'partition_spec', '_quota_name',
    )

    class Status(Enum):
        UNKNOWN = 'UNKNOWN'
        NORMAL = 'NORMAL'
        CLOSING = 'CLOSING'
        CLOSED = 'CLOSED'
        CANCELED = 'CANCELED'
        EXPIRED = 'EXPIRED'
        CRITICAL = 'CRITICAL'

    class UploadFile(serializers.JSONSerializableModel):
        file_name = serializers.JSONNodeField('FileName')
        file_length = serializers.JSONNodeField('FileLength')

    id = serializers.JSONNodeField('UploadID')
    status = serializers.JSONNodeField('Status',
                                       parse_callback=lambda v: VolumeUploadSession.Status(v.upper()))
    file_list = serializers.JSONNodesReferencesField(UploadFile, 'FileList')

    def __init__(
        self, client, volume, partition_spec, upload_id=None,
        compress_option=None, quota_name=None
    ):
        super(VolumeUploadSession, self).__init__()

        self._client = client
        self._compress_option = compress_option
        self._quota_name = quota_name
        self.project_name = volume.project.name
        self.volume_name = volume.name
        self.partition_spec = partition_spec

        if upload_id is None:
            self._init()
        else:
            self.id = upload_id
            self.reload()
        self._compress_option = compress_option

        logger.info("Tunnel session created: %r", self)
        if options.tunnel_session_create_callback:
            options.tunnel_session_create_callback(self)

    def __repr__(self):
        return (
            "<VolumeUploadSession id=%s project_name=%s volume_name=%s partition_spec=%s>"
            % (self.id, self.project_name, self.volume_name, self.partition_spec)
        )

    def resource(self, client=None, endpoint=None):
        endpoint = endpoint if endpoint is not None else (client or self._client).endpoint
        return endpoint + '/projects/%s/tunnel/uploads' % self.project_name

    def _init(self):
        headers = {'Content-Length': '0'}
        params = dict(
            type='volumefile',
            target='/'.join([self.project_name, self.volume_name, self.partition_spec]) + '/'
        )
        if self._quota_name is not None:
            params['quotaName'] = self._quota_name

        url = self.resource()
        resp = self._client.post(url, {}, params=params, headers=headers)
        if self._client.is_ok(resp):
            self.parse(resp, obj=self)
        else:
            e = TunnelError.parse(resp)
            raise e

    def reload(self):
        headers = {'Content-Length': '0'}
        params = {}

        url = self.resource() + '/' + str(self.id)
        resp = self._client.get(url, params=params, headers=headers)
        if self._client.is_ok(resp):
            self.parse(resp, obj=self)
        else:
            e = TunnelError.parse(resp)
            raise e

    @staticmethod
    def _format_file_name(file_name):
        buf = six.StringIO()
        if file_name and file_name[0] == '/':
            raise TunnelError("FileName cannot start with '/', file name is " + file_name)
        pre_slash = False
        for ch in file_name:
            if ch == '/':
                if not pre_slash:
                    buf.write(ch)
                pre_slash = True
            else:
                buf.write(ch)
                pre_slash = False
        return buf.getvalue()

    def open(self, file_name, compress=False, append=False):
        compress_option = self._compress_option or io.CompressOption()
        headers = {'Content-Type': 'test/plain', 'Transfer-Encoding': 'chunked', 'x-odps-tunnel-version': 4}
        params = {}

        if compress:
            if compress_option.algorithm == io.CompressOption.CompressAlgorithm.ODPS_ZLIB:
                headers['Content-Encoding'] = 'deflate'
            elif compress_option.algorithm != io.CompressOption.CompressAlgorithm.ODPS_RAW:
                raise TunnelError('invalid compression option')

        file_name = self._format_file_name(file_name)
        params['blockid'] = file_name
        if append:
            params['resume'] = ''
        if self._quota_name is not None:
            params['quotaName'] = self._quota_name

        url = self.resource() + '/' + self.id

        chunk_uploader = lambda data: self._client.post(url, data=data, params=params, headers=headers)
        option = compress_option if compress else None
        return VolumeWriter(self._client, chunk_uploader, option)

    def commit(self, files):
        if not files:
            raise ValueError('`files` not supplied')
        if isinstance(files, six.string_types):
            files = [files, ]
        formatted = [self._format_file_name(fn) for fn in files]

        self.reload()
        files_uploading = set(f.file_name for f in self.file_list)

        if len(files_uploading) != len(formatted):
            raise TunnelError("File number not match, server: %d, client: %d" % (len(files_uploading), len(formatted)))
        for fn in (fn for fn in formatted if fn not in files_uploading):
            raise TunnelError("File not exits on server, file name is " + fn)

        self._complete_upload()

    def _complete_upload(self):
        headers = {'Content-Length': '0'}
        params = {}
        if self._quota_name is not None:
            params['quotaName'] = self._quota_name

        url = self.resource() + '/' + self.id
        resp = self._client.put(url, {}, params=params, headers=headers)
        if self._client.is_ok(resp):
            self.parse(resp, obj=self)
        else:
            e = TunnelError.parse(resp)
            raise e


class VolumeWriter(object):
    CHUNK_SIZE = 512 * 1024

    def __init__(self, client, uploader, compress_option):
        self._client = client
        self._compress_option = compress_option
        self._req_io = io.RequestsIO(uploader, chunk_size=options.chunk_size)

        if compress_option is None:
            self._writer = self._req_io
        elif compress_option.algorithm == \
                io.CompressOption.CompressAlgorithm.ODPS_RAW:
            self._writer = self._req_io
        elif compress_option.algorithm == \
                io.CompressOption.CompressAlgorithm.ODPS_ZLIB:
            self._writer = io.DeflateOutputStream(self._req_io)
        else:
            raise errors.InvalidArgument('Invalid compression algorithm.')

        self._crc = Checksum(method='crc32')
        self._initialized = False
        self._chunk_offset = 0

    def _init_writer(self):
        chunk_bytes = struct.pack('>I', self.CHUNK_SIZE)
        self._writer.write(chunk_bytes)
        self._crc.update(chunk_bytes)
        self._chunk_offset = 0

    def write(self, buf, encoding='utf-8'):
        buf = to_binary(buf, encoding=encoding)
        if isinstance(buf, six.integer_types):
            buf = bytes(bytearray([buf, ]))
        elif isinstance(buf, six.BytesIO):
            buf = buf.getvalue()
        if not self._initialized:
            self._initialized = True
            self._init_writer()
            self._req_io.start()

        if not buf:
            raise IOError('Invalid data buffer!')
        processed = 0
        while processed < len(buf):
            if self._chunk_offset == self.CHUNK_SIZE:
                checksum = self._crc.getvalue()
                self._writer.write(struct.pack(CHECKSUM_PACKER, checksum))
                self._chunk_offset = 0
            else:
                size = self.CHUNK_SIZE - self._chunk_offset if len(buf) - processed > self.CHUNK_SIZE - self._chunk_offset\
                    else len(buf) - processed
                write_chunk = buf[processed:processed + size]
                self._writer.write(write_chunk)
                self._crc.update(write_chunk)
                processed += size
                self._chunk_offset += size

    def close(self):
        if not self._initialized:
            self._initialized = True
            self._init_writer()

        if self._chunk_offset != 0:
            checksum = self._crc.getvalue()
            self._writer.write(struct.pack(CHECKSUM_PACKER, checksum))
        self._writer.flush()
        result = self._req_io.finish()
        if result is None:
            raise TunnelError('No results returned in VolumeWriter.')
        if not self._client.is_ok(result):
            e = TunnelError.parse(result)
            raise e
        return result

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        # if an error occurs inside the with block, we do not commit
        if exc_val is not None:
            return
        self.close()


class VolumeFSWriter(VolumeWriter):
    def __init__(self, client, uploader, volume, path, compress_option):
        self._volume = volume
        self._path = path
        super(VolumeFSWriter, self).__init__(client, uploader, compress_option)

    def close(self):
        result = super(VolumeFSWriter, self).close()
        if 'x-odps-volume-sessionid' not in result.headers:
            raise TunnelError('No session id returned in response.')
        headers = {
            'x-odps-volume-fs-path': '/' + self._volume.name + '/' + self._path.lstrip('/'),
            'x-odps-volume-sessionid': result.headers.get('x-odps-volume-sessionid'),
        }
        commit_result = self._client.put(self._volume.resource(client=self._client), None, headers=headers)
        if not self._client.is_ok(commit_result):
            e = TunnelError.parse(commit_result)
            raise e
        return result



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright 1999-2022 Alibaba Group Holding Ltd.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from ..models import Table
from ..models.partition import Partition
from ..compat import six, izip
from ..types import CompositeMixin
from .expr.utils import get_attrs
from .expr.expressions import CollectionExpr
from .types import validate_data_type
from .backends.odpssql.types import odps_schema_to_df_schema
from .backends.pd.types import pd_to_df_schema, df_type_to_np_type, cast_composite_sequence
from .backends.sqlalchemy.types import sqlalchemy_to_df_schema

try:
    import pandas as pd
    has_pandas = True
except ImportError:
    has_pandas = False

try:
    import sqlalchemy
    has_sqlalchemy = True
except ImportError:
    has_sqlalchemy = False


class DataFrame(CollectionExpr):
    """
    Main entrance of PyODPS DataFrame.

    Users can initial a DataFrame by :class:`odps.models.Table`.

    :param data: ODPS table or pandas DataFrame
    :type data: :class:`odps.models.Table` or pandas DataFrame

    :Example:

    >>> df = DataFrame(o.get_table('my_example_table'))
    >>> df.dtypes
    odps.Schema {
      movie_id                            int64
      title                               string
      release_date                        string
      video_release_date                  string
      imdb_url                            string
      user_id                             int64
      rating                              int64
      unix_timestamp                      int64
      age                                 int64
      sex                                 string
      occupation                          string
      zip_code                            string
    }
    >>> df.count()
    100000
    >>>
    >>> # Do the `groupby`, aggregate the `movie_id` by count, then sort the count in a reversed order
    >>> # Finally we get the top 25 results
    >>> df.groupby('title').agg(count=df.movie_id.count()).sort('count', ascending=False)[:25]
    >>>
    >>> # We can use the `value_counts` to reach the same goal
    >>> df.movie_id.value_counts()[:25]
    """

    __slots__ = ()

    def __init__(self, *args, **kwargs):
        if len(args) == 1:
            data = args[0]
        else:
            data = kwargs.pop('_source_data', None)
            if data is None:
                raise ValueError('ODPS Table or pandas DataFrame should be provided.')

        if isinstance(data, Table):
            if '_schema' not in kwargs:
                kwargs['_schema'] = odps_schema_to_df_schema(data.table_schema)
            super(DataFrame, self).__init__(_source_data=data, **kwargs)
        elif isinstance(data, Partition):
            if '_schema' not in kwargs:
                kwargs['_schema'] = odps_schema_to_df_schema(data.table.table_schema)
            super(DataFrame, self).__init__(_source_data=data.parent.parent, **kwargs)
            self._proxy = self.copy().filter_parts(data)
        elif has_pandas and isinstance(data, pd.DataFrame):
            if 'schema' in kwargs and kwargs['schema']:
                schema = kwargs.pop('schema')
            elif '_schema' in kwargs:
                schema = kwargs.pop('_schema')
            else:
                unknown_as_string = kwargs.pop('unknown_as_string', False)
                as_type = kwargs.pop('as_type', None)
                if as_type:
                    data = data.copy()
                    data.is_copy = False
                    as_type = dict((k, validate_data_type(v)) for k, v in six.iteritems(as_type))

                    if not isinstance(as_type, dict):
                        raise TypeError('as_type must be dict')
                    for col_name, df_type in six.iteritems(as_type):
                        if col_name not in data:
                            raise ValueError('col(%s) does not exist in pd.DataFrame' % col_name)
                        try:
                            if isinstance(df_type, CompositeMixin):
                                data[col_name] = cast_composite_sequence(data[col_name], df_type)
                            else:
                                pd_type = df_type_to_np_type(df_type)
                                data[col_name] = data[col_name][data[col_name].notnull()].astype(pd_type)
                        except TypeError:
                            raise TypeError('Cannot cast col(%s) to data type: %s' % (col_name, df_type))
                schema = pd_to_df_schema(data, as_type=as_type,
                                         unknown_as_string=unknown_as_string)
            super(DataFrame, self).__init__(_source_data=data, _schema=schema, **kwargs)
        elif has_sqlalchemy and isinstance(data, sqlalchemy.Table):
            if '_schema' not in kwargs:
                kwargs['_schema'] = sqlalchemy_to_df_schema(data.c)
            super(DataFrame, self).__init__(_source_data=data, **kwargs)
        else:
            raise ValueError('Unknown type: %s' % data)

    def __setstate__(self, state):
        kv = dict(state)
        source_data = kv.pop('_source_data')
        kv.pop('_schema', None)
        self.__init__(source_data, **kv)

    def view(self):
        kv = dict((attr, getattr(self, attr)) for attr in get_attrs(self))
        data = kv.pop('_source_data')
        kv.pop('_schema', None)
        return type(self)(data, **kv)

    @staticmethod
    def batch_persist(dfs, tables, *args, **kwargs):
        """
        Persist multiple DataFrames into ODPS.

        :param dfs: DataFrames to persist.
        :param tables: Table names to persist to. Use (table, partition) tuple to store to a table partition.
        :param args: args for Expr.persist
        :param kwargs: kwargs for Expr.persist

        :Examples:
        >>> DataFrame.batch_persist([df1, df2], ['table_name1', ('table_name2', 'partition_name2')], lifecycle=1)
        """
        from .delay import Delay

        if 'async' in kwargs:
            kwargs['async_'] = kwargs['async']
        execute_keys = ('ui', 'async_', 'n_parallel', 'timeout', 'close_and_notify')
        execute_kw = dict((k, v) for k, v in six.iteritems(kwargs) if k in execute_keys)
        persist_kw = dict((k, v) for k, v in six.iteritems(kwargs) if k not in execute_keys)

        delay = Delay()
        persist_kw['delay'] = delay

        for df, table in izip(dfs, tables):
            if isinstance(table, tuple):
                table, partition = table
            else:
                partition = None
            df.persist(table, partition=partition, *args, **persist_kw)

        return delay.execute(**execute_kw)

    @property
    def data(self):
        return self._source_data



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright 1999-2022 Alibaba Group Holding Ltd.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from ..core import Iterable, LazyLoad
from ...compat import six
from ... import serializers, errors


class User(LazyLoad):
    id = serializers.XMLNodeField('ID')
    display_name = serializers.XMLNodeField('DisplayName')
    comment = serializers.XMLNodeField('Comment')

    def _name(self):
        return self.id

    def reload(self):
        if self._getattr('id') is None:
            resp = self._client.get(self.parent.resource() + '/' + self._encode(self.display_name),
                                    params=dict(type='displayname'))
            self.parse(self._client, resp, obj=self)
        else:
            resp = self._client.get(self.resource())
            self.parse(self._client, resp, obj=self)

    @property
    def roles(self):
        from .roles import Roles
        params = dict(roles='', type='displayname')
        resp = self._client.get(self.resource(), params=params)
        roles = Roles.parse(self._client, resp, parent=self.project)
        roles._iter_local = True
        return roles

    def grant_role(self, name):
        from .roles import Role
        if isinstance(name, Role):
            name = name.name
        self.project.run_security_query('grant %s to %s' % (name, self.display_name))

    def revoke_role(self, name):
        from .roles import Role
        if isinstance(name, Role):
            name = name.name
        self.project.run_security_query('revoke %s from %s' % (name, self.display_name))


class Users(Iterable):
    __slots__ = '_iter_local',
    users = serializers.XMLNodesReferencesField(User, 'User')

    def __init__(self, **kw):
        self._iter_local = False
        super(Users, self).__init__(**kw)

    def _get(self, item):
        return User(client=self._client, parent=self, display_name=item)

    def __contains__(self, item):
        if isinstance(item, six.string_types):
            user = self._get(item)
        elif isinstance(item, User):
            user = item
        else:
            return False

        if not self._iter_local:
            try:
                user.reload()
                return True
            except errors.NoSuchObject:
                return False
        else:
            return any(u.display_name == user.display_name for u in self.users)

    def __iter__(self):
        return self.iterate()

    @property
    def project(self):
        return self.parent

    def create(self, name):
        self.project.run_security_query('add user %s' % name)
        return User(client=self._client, parent=self, display_name=name)

    def iterate(self):
        """
        :return:
        """
        if not self._iter_local:
            params = dict()
            url = self.resource()
            resp = self._client.get(url, params=params)

            Users.parse(self._client, resp, obj=self)

        for user in self.users:
            yield user

    def delete(self, name):
        if isinstance(name, User):
            name = name.display_name

        self.project.run_security_query('remove user %s' % name)



# encoding: utf-8
# Copyright 1999-2022 Alibaba Group Holding Ltd.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from ..compat import Enum


class FieldRole(Enum):
    FEATURE = 'FEATURE'
    LABEL = 'LABEL'
    WEIGHT = 'WEIGHT'
    GROUP_ID = 'GROUP_ID'
    PREDICTED_CLASS = 'PREDICTED_CLASS'
    PREDICTED_SCORE = 'PREDICTED_SCORE'
    PREDICTED_VALUE = 'PREDICTED_VALUE'
    PREDICTED_DETAIL = 'PREDICTED_DETAIL'


class FieldContinuity(Enum):
    CONTINUOUS = 'CONTINUOUS'
    DISCRETE = 'DISCRETE'


class PortType(Enum):
    DATA = 'DATA'
    MODEL = 'MODEL'


class PortDirection(Enum):
    INPUT = 'INPUT'
    OUTPUT = 'OUTPUT'



import sys
from types import TracebackType

from . import Frame
from . import Traceback

if sys.version_info[0] >= 3:
    import copyreg
else:
    import copy_reg as copyreg


def unpickle_traceback(tb_frame, tb_lineno, tb_next):
    ret = object.__new__(Traceback)
    ret.tb_frame = tb_frame
    ret.tb_lineno = tb_lineno
    ret.tb_next = tb_next
    return ret.as_traceback()


def pickle_traceback(tb):
    return unpickle_traceback, (Frame(tb.tb_frame), tb.tb_lineno, tb.tb_next and Traceback(tb.tb_next))


def unpickle_exception(func, args, cause, tb):
    inst = func(*args)
    inst.__cause__ = cause
    inst.__traceback__ = tb
    return inst


def pickle_exception(obj):
    # All exceptions, unlike generic Python objects, define __reduce_ex__
    # __reduce_ex__(4) should be no different from __reduce_ex__(3).
    # __reduce_ex__(5) could bring benefits in the unlikely case the exception
    # directly contains buffers, but PickleBuffer objects will cause a crash when
    # running on protocol=4, and there's no clean way to figure out the current
    # protocol from here. Note that any object returned by __reduce_ex__(3) will
    # still be pickled with protocol 5 if pickle.dump() is running with it.
    rv = obj.__reduce_ex__(3)
    if isinstance(rv, str):
        raise TypeError("str __reduce__ output is not supported")
    assert isinstance(rv, tuple) and len(rv) >= 2

    return (unpickle_exception, rv[:2] + (obj.__cause__, obj.__traceback__)) + rv[2:]


def _get_subclasses(cls):
    # Depth-first traversal of all direct and indirect subclasses of cls
    to_visit = [cls]
    while to_visit:
        this = to_visit.pop()
        yield this
        to_visit += list(this.__subclasses__())


def install(*exc_classes_or_instances):
    copyreg.pickle(TracebackType, pickle_traceback)

    if sys.version_info[0] < 3:
        # Dummy decorator?
        if len(exc_classes_or_instances) == 1:
            exc = exc_classes_or_instances[0]
            if isinstance(exc, type) and issubclass(exc, BaseException):
                return exc
        return

    if not exc_classes_or_instances:
        for exception_cls in _get_subclasses(BaseException):
            copyreg.pickle(exception_cls, pickle_exception)
        return

    for exc in exc_classes_or_instances:
        if isinstance(exc, BaseException):
            while exc is not None:
                copyreg.pickle(type(exc), pickle_exception)
                exc = exc.__cause__
        elif isinstance(exc, type) and issubclass(exc, BaseException):
            copyreg.pickle(exc, pickle_exception)
            # Allow using @install as a decorator for Exception classes
            if len(exc_classes_or_instances) == 1:
                return exc
        else:
            raise TypeError(
                "Expected subclasses or instances of BaseException, got %s"
                % (type(exc))
            )
